@@ -2,31 +2,39 @@
 
 # chem - a groff preprocessor for producing chemical structure diagrams
 
-my $copyright = 'Copyright (C) 2006-2014, 2022'
-                . ' Free Software Foundation, Inc.';
-# Written by Bernd Warken <groff-bernd.warken-72@web.de>.
+# Source file position: <groff-source>/contrib/chem/chem.pl
+# Installed position: <prefix>/bin/chem
 
-# This file is part of 'chem', which is part of 'groff'.
+# Copyright (C) 2006, 2009 Free Software Foundation, Inc.
+# Written by Bernd Warken.
 
-# 'groff' is free software; you can redistribute it and/or modify it
-# under the terms of the GNU General Public License (GPL) version 2 as
-# published by the Free Software Foundation.
+# This file is part of `chem', which is part of `groff'.
 
-# 'groff' is distributed in the hope that it will be useful, but
+# `groff' is free software; you can redistribute it and/or modify it
+# under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+
+# `groff' is distributed in the hope that it will be useful, but
 # WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # General Public License for more details.
 
-# The GPL2 license text is available in the internet at
-# <http://www.gnu.org/licenses/gpl-2.0.html>.
+# You should have received a copy of the GNU General Public License
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 ########################################################################
 # settings
 ########################################################################
 
-my $chem_version = '1.0.6';
-my $groff_version = 'DEVELOPMENT';
+my $Program_Version = '0.3.1';
+my $Last_Update = '03 Jan 2009';
 
+# this setting of the groff version is only used before make is run,
+# otherwise @VERSION@ will set it.
+my $Groff_Version_Preset='1.20preset';
+
+# test on Perl version
 require v5.6;
 
 
@@ -44,27 +52,41 @@ use File::Spec;
 # $Bin is the directory where this script is located
 use FindBin;
 
-my $chem;
+my $Chem_Name;
+my $Groff_Version;
 my $File_chem_pic;
+my $File_pic_tmac;
 
-my $is_in_source_tree;
-{
-  $is_in_source_tree = 1 if '@VERSION@' eq '@' . 'VERSION' . '@';
-}
+BEGIN {
+  {
+    my $before_make;		# script before run of `make'
+    {
+      my $at = '@';
+      $before_make = 1 if '@VERSION@' eq "${at}VERSION${at}";
+    }
 
-my %makevar;
+    my %at_at;
 
-if ($is_in_source_tree) {
-  my $chem_dir = $FindBin::Bin;
-  $makevar{'G'} = '';
-  $File_chem_pic = File::Spec->catfile($chem_dir, 'chem.pic');
-  $chem = 'chem';
-} else {
-  $groff_version = '@VERSION@';
-  $makevar{'G'} = '@g@';
-  $makevar{'PICDIR'} = '@PICDIR@';
-  $File_chem_pic = File::Spec->catfile($makevar{'PICDIR'}, 'chem.pic');
-  $chem = $makevar{'G'} . 'chem';
+    if ($before_make) {
+      my $chem_dir = $FindBin::Bin;
+      $at_at{'BINDIR'} = $chem_dir;
+      $at_at{'G'} = '';
+      $File_chem_pic = File::Spec->catfile($chem_dir, 'chem.pic');
+      $File_pic_tmac = File::Spec->catfile($chem_dir, '..', 'pic.tmac');
+      $Groff_Version = '';
+      $Chem_Name = 'chem';
+    } else {
+      $Groff_Version = '@VERSION@';
+      $at_at{'BINDIR'} = '@BINDIR@';
+      $at_at{'G'} = '@g@';
+      $at_at{'PICDIR'} = '@PICDIR@';
+      $at_at{'TMACDIR'} = '@MACRODIR@';
+      $File_chem_pic =
+	File::Spec->catfile($at_at{'PICDIR'}, 'chem.pic');
+      $File_pic_tmac = File::Spec->catfile($at_at{'TMACDIR'}, 'pic.tmac');
+      $Chem_Name = $at_at{'G'} . 'chem';
+    }
+  }
 }
 
 
@@ -192,6 +214,10 @@ my $Line = '';
     my @stdin = ();
     my $stdin = 0;
 
+    # for centralizing the pic code
+    open TMAC, "<$File_pic_tmac" and print <TMAC>;
+    close TMAC;
+
     foreach (@ARGV) {
       $count_minus++ if /^-$/;
     }
@@ -313,9 +339,9 @@ my $Line = '';
       return 1;
     }
     if ($line =~ /^[\.']\s*cstart\s*$/) {
-      # line: '.cstart'
+      # line: `.cstart'
       if ($is_chem) {
-	&error("additional '.cstart'; chem is already active.");
+	&error("additional `.cstart'; chem is already active.");
 	return 1;
       }
       unless ($is_pic) {
@@ -328,10 +354,10 @@ my $Line = '';
     }
 ### main_line()
     if ($line =~ /^\s*begin\s+chem\s*$/) {
-      # line: 'begin chem'
+      # line: `begin chem'
       if ($is_pic) {
 	if ($is_chem) {
-	  &error("additional 'begin chem'; chem is already active.");
+	  &error("additional `begin chem'; chem is already active.");
 	  return 1;
 	}
 	$is_chem = 'begin chem';
@@ -342,9 +368,9 @@ my $Line = '';
       return 1;
     }
     if ($line =~ /^[\.']\s*cend\s*/) {
-      # line '.cend'
+      # line `.cend'
       if ($is_chem) {
-	&error("you end chem with '.cend', but started it with 'begin chem'.")
+	&error("you end chem with `.cend', but started it with `begin chem'.")
 	  if $is_chem eq 'begin chem';
 	if ($is_pic eq 'by chem') {
 	  &print_pe();
@@ -357,9 +383,9 @@ my $Line = '';
       return 1;
     }
     if ($line =~ /^\s*end\s*$/) {
-      # line: 'end'
+      # line: `end'
       if ($is_chem) {
-	&error("you end chem with 'end', but started it with '.cstart'.")
+	&error("you end chem with `end', but started it with `.cstart'.")
 	  if $is_chem eq '.cstart';
 	if ($is_pic eq 'by chem') {
 	  &print_pe();
@@ -384,7 +410,7 @@ my $Line = '';
     }
 
     if ($Words[0] eq 'pic') {
-      # pic pass-through
+      # pic pass-thru
       return 1 if $#Words == 0;
       my $s = $line;
       $s =~ /^\s*pic\s*(.*)$/;
@@ -397,10 +423,10 @@ my $Line = '';
 
     if ($Words[0] eq 'textht') {
       if ($#Words == 0) {
-	&error("'textht' needs a single argument.");
+	&error("`textht' needs a single argument.");
 	return 0;
       }
-      &error("only the last argument is taken for 'textht', " .
+      &error("only the last argument is taken for `textht', " .
 	     "all others are ignored.")
 	unless $#Words <= 1 or ($#Words == 2 && $Words[1] =~ /^=/);
       $Params{'textht'} = $Words[$#Words];
@@ -409,10 +435,10 @@ my $Line = '';
 ### main_line()
     if ($Words[0] eq 'cwid') {	# character width
       if ($#Words == 0) {
-	&error("'cwid' needs a single argument.");
+	&error("`cwid' needs a single argument.");
 	return 0;
       }
-      &error("only the last argument is taken for 'cwid', " .
+      &error("only the last argument is taken for `cwid', " .
 	     "all others are ignored.")
 	unless $#Words <= 1 or ($#Words == 2 && $Words[1] =~ /^=/);
       $Params{'cwid'} = $Words[$#Words];
@@ -420,10 +446,10 @@ my $Line = '';
     }
     if ($Words[0] eq 'db') {	# bond length
       if ($#Words == 0) {
-	&error("'db' needs a single argument.");
+	&error("`db' needs a single argument.");
 	return 0;
       }
-      &error("only the last argument is taken for 'db', " .
+      &error("only the last argument is taken for `db', " .
 	     "all others are ignored.")
 	unless $#Words <= 1 or ($#Words == 2 && $Words[1] =~ /^=/);
       $Params{'db'} = $Words[$#Words];
@@ -432,10 +458,10 @@ my $Line = '';
     if ($Words[0] eq 'size') {	# size for all parts of the whole diagram
       my $size;
       if ($#Words == 0) {
-	&error("'size' needs a single argument.");
+	&error("`size' needs a single argument.");
 	return 0;
       }
-      &error("only the last argument is taken for 'size', " .
+      &error("only the last argument is taken for `size', " .
 	     "all others are ignored.")
 	unless $#Words <= 1 or ($#Words == 2 && $Words[1] =~ /^=/);
       if ($Words[$#Words] <= 4) {
@@ -554,7 +580,7 @@ my $Line = '';
     return 1 unless $line;
 #    print STDERR "# $Line\n";
 #    &error('This is not a chem command.  To include a command for pic, ' .
-#           "add 'pic' as the first word to the command.");
+#	   "add `pic' as the first word to the command.");
     print $line, "\n";
     $Last_Type = $Types{'OTHER'};
     1;
@@ -612,7 +638,7 @@ sub bond {
   $moiety = '';
   for ($i = 1; $i <= $#Words; $i++) {
     if ($Words[$i] eq ';') {
-      &error("a colon ';' must be followed by a space and a single word.")
+      &error("a colon `;' must be followed by a space and a single word.")
        if $i != $#Words - 1;
       $moiety = $Words[$i + 1] if $#Words > $i;
       $#Words = $i - 1;
@@ -1194,38 +1220,55 @@ sub setparams {
 } # setparams()
 
 
+##########
+# usage()
+#
+# Print usage information for --help.
+#
 sub usage {
+  print "\n";
+  &version();
   print <<EOF;
-usage: $chem [file ...]
-usage: $chem { -h | --help | -v | --version }
 
-$chem is a groff preprocessor for producing chemical structure
-diagrams.  It produces input for the $makevar{'G'}pic preprocessor.  If
-no file operands are given, or if file is "-", the standard input stream
-is read.
+Usage: $Chem_Name [option]... [filespec]...
+
+$Chem_Name is a groff preprocessor for producing chemical structure
+diagrams.  The output suits to the pic preprocessor.
+
+"filespec" is one of
+  "filename"       name of a readable file
+  "-"              for standard input
+
+All available options are
+
+-h --help         print this usage message.
+-v --version      print version information.
 
-Options:
- -h, --help     Display this message and exit.
- -v, --version  Display version information and exit.
 EOF
-}
+} # usage()
 
 
+##########
+# version()
+#
+# Get version information from version.sh and print a text with this.
+#
 sub version {
+  $Groff_Version = $Groff_Version_Preset unless $Groff_Version;
+  my $year = $Last_Update;
+  $year =~ s/^.* //;
   print <<EOF;
-$chem (groff $groff_version) $chem_version
-$copyright
-License GPLv2: GNU GPL version 2
-<https://www.gnu.org/licenses/old-licenses/gpl-2.0.en.html>
-This is free software: you are free to change and redistribute it.
-There is NO WARRANTY, to the extent permitted by law.
+$Chem_Name $Program_Version of $Last_Update (Perl version)
+is part of groff version $Groff_Version.
+Copyright (C) $year Free Software Foundation, Inc.
+GNU groff and chem come with ABSOLUTELY NO WARRANTY.
+You may redistribute copies of groff and its subprograms
+under the terms of the GNU General Public License.
 EOF
-}
+} # version()
 
 1;
-
+### Emacs settings
 # Local Variables:
-# fill-column: 72
 # mode: CPerl
 # End:
-# vim: set cindent noexpandtab shiftwidth=2 softtabstop=2 textwidth=72:
